import os
import unittest
import StringIO
from Globals import InitializeClass, package_home
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFPlone.utils import _createObjectByType

from Products.Mlango.tests.base import MlangoTestCase
import Products.Mlango.content.MlangoViewlet


def loadImage(name, size=0):
    """Load image from testing directory
    """
    path = os.path.join(package_home(globals()), 'input', name)
    fd = open(path, 'rb')
    data = fd.read()
    fd.close()
    return data
    
TEST_GIF = loadImage('test.gif')



class TestContentTypes(MlangoTestCase):

    def afterSetUp(self):
        self.setRoles(['Manager', 'Member'])

        self.workflow = getToolByName(self.portal, 'portal_workflow')
        self.portalRoot =  self.portal.portal_url.getPortalObject()
        
        self.mlangoDashboard = "MlangoDashboard"
        self.mlangoViewlet = "MlangoViewlet"
        self.mlangoRSSViewlet = "MlangoRSSViewlet"
        self.mlangoCustomViewlet = "MlangoCustomViewlet"
        
        self.types = [self.mlangoViewlet, self.mlangoRSSViewlet, self.mlangoCustomViewlet]
        
    def testCreateTypes(self):
        """ Create content types and test workflow
            presence.
        """
        for type in self.types:
            typeId = self.portal.generateUniqueId()
            typeTitle = "%s title" % type
            
            _createObjectByType(type, self.portal, typeId)
            
            obj = getattr(self.portal, typeId)
            obj.setTitle(typeTitle)
            
            self.assertEqual(obj.getId(), typeId)
            self.assertEqual(obj.Title(), typeTitle)
    
    def testMlangoDashboard(self):
        """
        """
        dashboardId = 'dashboard'
        dashboardTitle = 'Mlango dashboard'
        
        _createObjectByType(self.mlangoDashboard, self.portal, dashboardId)
        
        dashboard = getattr(self.portal, dashboardId)
        
        # test fields in schema
        dashboard.setCols(4)
        self.assertEqual(dashboard.getCols(), 4)
        
        flow = self.workflow.getWorkflowsFor(dashboard)[0]
        self.failUnless(flow.id == 'dashboard_workflow')
    
    
    def testMlangoViewlet(self):
        """ Test viewlet
        """
        viewletId = "myViewlet"
        viewletHelp = "This is a help string"
             
        _createObjectByType(self.mlangoViewlet, self.portal, viewletId)
        
        contentIndex = os.path.join(\
            os.path.dirname(Products.Mlango.content.MlangoViewlet.__file__))
        
        viewlet = getattr(self.portal, viewletId)
        
        # test fields in schema
        viewlet.setViewleticon(TEST_GIF, mimetype='image/gif', filename='test.gif')
        self.failUnless(viewlet.getViewleticon())
        
        viewlet.setHelp(viewletHelp)
        self.assertEqual(viewlet.getHelp(), viewletHelp)
        
        viewlet.setFixed(-1)
        self.assertEqual(viewlet.getFixed(), -1)
      
        self.failUnless(viewlet.returnTemplate())
        self.failUnless(viewlet.returnContentTemplate()) 


def test_suite():
    """This sets up a test suite that actually runs the tests in the class
    above
    """
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestContentTypes))
    return suite



# -*- coding: utf-8 -*-

from zope.interface import Interface

##code-section HEAD
##/code-section HEAD

class IMlangoDashboard(Interface):
    """Marker interface for .MlangoDashboard.MlangoDashboard
    """

class IMlangoViewlet(Interface):
    """Marker interface for .MlangoViewlet.MlangoViewlet
    """

class IMlangoRSSViewlet(Interface):
    """Marker interface for .MlangoRSSViewlet.MlangoRSSViewlet
    """

class IMlangoCustomViewlet(Interface):
    """Marker interface for .MlangoCustomViewlet.MlangoCustomViewlet
    """

##code-section FOOT
##/code-section FOOT


# -*- coding: utf-8 -*-
#
# File: MlangoCustomViewlet.py
#
# Copyright (c) 2008 by Goldmund-Wyldebeast & Wunderliebe
# Generator: ArchGenXML Version 2.1
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#

__author__ = """Goldmund-Wyldebeast & Wunderliebe <info@gw20e.com>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from zope.interface import implements
import interfaces
from Products.Mlango.content.MlangoViewlet import MlangoViewlet
from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin

from Products.Mlango.config import *

# additional imports from tagged value 'import'
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

##code-section module-header #fill in your manual code here
##/code-section module-header

schema = Schema((

    TextField(
        name='content',
        widget=TextAreaWidget(
            cols=20,
            rows=100,
            description="Enter the HTML code you want this viewlet to display. TAL and METAL name space are not available.",
            label='Content',
            label_msgid='Mlango_label_content',
            description_msgid='Mlango_help_content',
            i18n_domain='Mlango',
        ),
        required=True,
    ),

),
)

##code-section after-local-schema #fill in your manual code here
##/code-section after-local-schema

MlangoCustomViewlet_schema = BaseSchema.copy() + \
    getattr(MlangoViewlet, 'schema', Schema(())).copy() + \
    schema.copy()

##code-section after-schema #fill in your manual code here
##/code-section after-schema

class MlangoCustomViewlet(BaseContent, MlangoViewlet, BrowserDefaultMixin):
    """MlangoViewlet Class which can have custom content as content.
    """
    security = ClassSecurityInfo()

    implements(interfaces.IMlangoCustomViewlet)

    meta_type = 'MlangoCustomViewlet'
    _at_rename_after_creation = True

    schema = MlangoCustomViewlet_schema

    ##code-section class-header #fill in your manual code here
    _contenttemplate = ViewPageTemplateFile('MlangoCustomViewlet.pt')
    ##/code-section class-header

    # Methods


registerType(MlangoCustomViewlet, PROJECTNAME)
# end of class MlangoCustomViewlet

##code-section module-footer #fill in your manual code here
##/code-section module-footer






from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone.memoize.compress import xhtml_compress

class ViewletFeeder:
    """ This Zope3 view enables viewlets easily to return show their entire frame (+content),
    or just their content! Down side is that the "Classic Viewlet" (using macro's of old portlets),
    is unable to work in such a context due to lack of 'here' or 'here_url'.
    """
    def feeder(self):
        """ Retrieve the template from the viewlet (same for all),
        and then use xhtml_compress to view it.
        """
        template = self.context.returnTemplate()
        return xhtml_compress(template())
    
    def contentfeeder(self):
        """ Retrieve the contenttemplate from the viewlet (unique for each content type of viewlet),
        and then use xhtml_compress to view it.
        This view is called in the template to draw the content of a viewlet.
        """
        template = self.context.returnContentTemplate()
        return xhtml_compress(template())



"""This is an integration "unit" test. It uses PloneTestCase, but does not
use doctest syntax.

You will find lots of examples of this type of test in CMFPlone/tests, for 
example.
"""

import unittest

from Products.ResourceRegistries.config import CSSTOOLNAME, JSTOOLNAME

from Products.Mlango.tests.base import MlangoTestCase
from Products.CMFCore.utils import getToolByName

from Products.Mlango.config import TOOL_ID
from Products.Mlango.tools.MlangoTool import MlangoTool



class TestInstallation(MlangoTestCase):
    """ Tests installation of Mlango
    """
    
    def afterSetUp(self):
        self.tool = getattr(self.portal.aq_explicit, TOOL_ID)
        self.ttool = getattr(self.portal.aq_explicit, 'portal_types')
        self.cat = getattr(self.portal.aq_explicit, 'portal_catalog')
        self.controlpanel = getToolByName(self.portal, "portal_controlpanel")
        self.csstool = getToolByName(self.portal, CSSTOOLNAME)
        self.jstool = getToolByName(self.portal, JSTOOLNAME)
        
        self.configlets = ['MlangoTool',]


    def testToolInstalled(self):
        """ Mlango tool
        """
        t = getToolByName(self.portal, TOOL_ID, None)
        self.failUnless(t, t)
        self.failUnless(isinstance(t, MlangoTool), t.__class__)
        self.failUnlessEqual(t.meta_type, 'MlangoTool')
        self.failUnlessEqual(t.getId(), TOOL_ID)
        
        

    def testSkinInstalled(self):
        """ Portal skins
        """
        stool = getattr(self.portal.aq_explicit, 'portal_skins')
        ids = stool.objectIds()
        self.failUnless('mlango_images' in ids, ids)
        self.failUnless('mlango_styles' in ids, ids)        
        self.failUnless('mlango_templates' in ids, ids)        

    def testInstalledAllTypes(self):
        """ Portal types
        """
        # test that all types are installed well
        ttool = self.ttool
        ids = ('MlangoDashboard', 'MlangoViewlet', 'MlangoRSSViewlet',
               'MlangoCustomViewlet')

        tids = ttool.objectIds()
        for id in ids:
            self.failUnless(id in tids, (id, tids))
            tinfo = ttool[id]
            self.failUnless(tinfo.product == 'Mlango', tinfo.product)

    def testControlPanel(self):
        """ Plone control panel
        """
        for title in self.configlets:
            self.failUnless(title in [a.getAction(self)['id']
                                   for a in self.controlpanel.listActions()],
                            "Missing configlet with id '%s'" % title)
        
    def testCssToolExists(self):
        """ CSS tool
        """
        self.failUnless(CSSTOOLNAME in self.portal.objectIds())
       
    def testCssRegistry(self):
        """ CSS registry
        """
        installedStylesheetIds = self.csstool.getResourceIds()
        expected = ['mlango.css', 'borders.css']
        for e in expected:
            self.failUnless(e in installedStylesheetIds, e)

    def testJsToolExists(self):
        """ JS tool
        """
        self.failUnless(JSTOOLNAME in self.portal.objectIds())
    
    def testJsRegistry(self):
        """ JS registry
        """
        installedScriptIds = self.jstool.getResourceIds()
        expected = ['jquery.js',]
        for e in expected:
            self.failUnless(e in installedScriptIds, e)
        
    def testJsInstertedInPage(self):
        page = self.portal.index_html()
        self.failUnless("" in page)
        

def test_suite():
    """This sets up a test suite that actually runs the tests in the class
    above
    """
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestInstallation))
    return suite



# -*- coding: utf-8 -*-
#
# File: Mlango.py
#
# Copyright (c) 2008 by Goldmund-Wyldebeast & Wunderliebe
# Generator: ArchGenXML Version 2.1
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#

__author__ = """Goldmund-Wyldebeast & Wunderliebe <info@gw20e.com>"""
__docformat__ = 'plaintext'


# Product configuration.
#
# The contents of this module will be imported into __init__.py, the
# workflow configuration and every content type module.
#
# If you wish to perform custom configuration, you may put a file
# AppConfig.py in your product's root directory. The items in there
# will be included (by importing) in this file if found.

from Products.CMFCore.permissions import setDefaultRoles
##code-section config-head #fill in your manual code here
##/code-section config-head


PROJECTNAME = "Mlango"

# Permissions
DEFAULT_ADD_CONTENT_PERMISSION = "Add portal content"
setDefaultRoles(DEFAULT_ADD_CONTENT_PERMISSION, ('Manager', 'Owner'))
ADD_CONTENT_PERMISSIONS = {
    'MlangoDashboard': 'Mlango: Add MlangoDashboard',
    'MlangoViewlet': 'Mlango: Add MlangoViewlet',
    'MlangoRSSViewlet': 'Mlango: Add MlangoRSSViewlet',
    'MlangoCustomViewlet': 'Mlango: Add MlangoCustomViewlet',
}

setDefaultRoles('Mlango: Add MlangoDashboard', ('Manager','Owner'))
setDefaultRoles('Mlango: Add MlangoViewlet', ('Manager','Owner'))
setDefaultRoles('Mlango: Add MlangoRSSViewlet', ('Manager','Owner'))
setDefaultRoles('Mlango: Add MlangoCustomViewlet', ('Manager','Owner'))

product_globals = globals()

# Dependencies of Products to be installed by quick-installer
# override in custom configuration
DEPENDENCIES = []

# Dependend products - not quick-installed - used in testcase
# override in custom configuration
PRODUCT_DEPENDENCIES = []

##code-section config-bottom #fill in your manual code here

VIEWLET_DEFAULTS = {'col_0':'','col_1':''}
TOOL_ID = 'portal_mlangotool'

VIEWLETS = (
            {'id': 'help',
             'title': 'Info',
             'icon': 'help.gif',
             'help': 'This viewlet contains Mlango relevant information.',
             'fixed': 1,
             },
             {'id': 'poweredby',
             'title': 'Power',
             'icon': 'user.gif',
             'help': 'Mlang is brought to you by Goldmund-Wyldebeast&Wunderliebe',
             'macro': 'here/powered_by_viewlet/macros/body',
             'fixed': -1,
             },
            )
DEBUG = True
##/code-section config-bottom


# Load custom configuration not managed by archgenxml
try:
    from Products.Mlango.AppConfig import *
except ImportError:
    pass



#!/usr/bin/python
# Copyright (C) 2008  Goldmund, Wyldebeast & Wunderliebe
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

class Feedparser:
    
      
    def parse(self, url, entry_fields=['title', 'link'], feed_fields=['title']):
    
        """
        Parse given feed, and create a simple dictionary with a feed and a entries
        field. 
        """
        try:
            import feedparser
        except:
            self.context.plone_log('Unable to import feedparser')
            from Products.Mlango import feedparser
        
        data = feedparser.parse(url) # feedparser.FeedParserDict
      
        entries = []
        feed = {}
      
        for f in feed_fields:
            feed[f] = data.feed.get(f, '')
    
        # very simple one for now
        for e in data.entries:
            entry = {}
            for f in entry_fields:
                self.context.plone_log('ENTRY f: %s  e.get: %s ' % (f, e.get(f, '').encode('utf8')))
                entry[f] = e.get(f, '').encode('utf8')         
            entries.append(entry)
    
    
        return {'feed':feed, 'entries':entries}



# -*- coding: utf-8 -*-

from zope.interface import Interface

##code-section HEAD
##/code-section HEAD

class IMlangoTool(Interface):
    """Marker interface for .MlangoTool.MlangoTool
    """

##code-section FOOT
##/code-section FOOT


# -*- coding: utf-8 -*-
#
# File: UserSettingsStorage.py
#
# Copyright (c) 2008 by Goldmund-Wyldebeast & Wunderliebe
# Generator: ArchGenXML Version 2.1
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#

__author__ = """Goldmund-Wyldebeast & Wunderliebe <info@gw20e.com>"""
__docformat__ = 'plaintext'

##code-section module-header #fill in your manual code here
from Products.CMFCore.permissions import *

##/code-section module-header

from Products.Mlango.config import VIEWLET_DEFAULTS
from zope.interface import implements
from Globals import PersistentMapping
from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin


class UserSettingsStorage(PersistentMapping):
    """
    """

    ##code-section class-header_UserSettingsStorage #fill in your manual code here
    ##/code-section class-header_UserSettingsStorage

    def getUserSettings(self, user_id,dashboard_id, prefix=None):


        if not prefix:
            return self.get(user_id, {}).get(dashboard_id, VIEWLET_DEFAULTS)
        else:
            data = {}
            for k in self.get(user_id, {}).get(dashboard_id, {}).keys():
                if k.startswith(prefix):
                    data[k] = self[user_id][dashboard_id][k]
            return data

    def setUserSettings(self, user_id, dashboard_id, data):

        self[user_id][dashboard_id] = data;
        self._p_changed = 1 # why is this needed in PersistentMapping?

    def updateUserSettings(self, user_id, dashboard_id, data):

        if not self.has_key(user_id):
            self[user_id] = {};

        if not self[user_id].has_key(dashboard_id):
            self[user_id][dashboard_id] = {}

        for k in data.keys():
            self[user_id][dashboard_id][k] = data[k]

        self._p_changed = 1 # why is this needed in PersistentMapping?

    def listBoardsForUser(self, user_id):
            return self.get(user_id, None)

    def listAllBoardsWithSettings(self):
        """This method goes by all boards which users have settings for,
        and adds them to one big list which it returns.
        """
        allBoards = []
        for value in self.itervalues():
            allBoards = [key for key in value.keys() if key not in allBoards] + allBoards
        return allBoards


##code-section module-footer #fill in your manual code here
##/code-section module-footer





# -*- coding: utf-8 -*-
#
# File: setuphandlers.py
#
# Copyright (c) 2008 by Goldmund-Wyldebeast & Wunderliebe
# Generator: ArchGenXML Version 2.1
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#

__author__ = """Goldmund-Wyldebeast & Wunderliebe <info@gw20e.com>"""
__docformat__ = 'plaintext'


import logging
logger = logging.getLogger('Mlango: setuphandlers')
from Products.Mlango.config import PROJECTNAME
from Products.Mlango.config import DEPENDENCIES
import os
from Products.CMFCore.utils import getToolByName
import transaction
##code-section HEAD
##/code-section HEAD

def isNotMlangoProfile(context):
    return context.readDataFile("Mlango_marker.txt") is None

def setupHideToolsFromNavigation(context):
    """hide tools"""
    if isNotMlangoProfile(context): return 
    # uncatalog tools
    site = context.getSite()
    toolnames = ['portal_mlangotool']
    portalProperties = getToolByName(site, 'portal_properties')
    navtreeProperties = getattr(portalProperties, 'navtree_properties')
    if navtreeProperties.hasProperty('idsNotToList'):
        for toolname in toolnames:
            try:
                portal[toolname].unindexObject()
            except:
                pass
            current = list(navtreeProperties.getProperty('idsNotToList') or [])
            if toolname not in current:
                current.append(toolname)
                kwargs = {'idsNotToList': current}
                navtreeProperties.manage_changeProperties(**kwargs)

def fixTools(context):
    """do post-processing on auto-installed tool instances"""
    if isNotMlangoProfile(context): return 
    site = context.getSite()
    tool_ids=['portal_mlangotool']
    for tool_id in tool_ids:
	    if hasattr(site, tool_id):
	        tool=site[tool_id]
	        tool.initializeArchetype()



def updateRoleMappings(context):
    """after workflow changed update the roles mapping. this is like pressing
    the button 'Update Security Setting' and portal_workflow"""
    if isNotMlangoProfile(context): return 
    wft = getToolByName(context.getSite(), 'portal_workflow')
    wft.updateRoleMappings()


def postInstall(context):
    """Called as at the end of the setup process. """
    # the right place for your custom code
    if isNotMlangoProfile(context): return 
    site = context.getSite()


##code-section FOOT
##/code-section FOOT



"""This is a a functional doctest test. It uses PloneTestCase and doctest
syntax. In the test itself, we use zope.testbrowser to test end-to-end
functionality, including the UI.

One important thing to note: zope.testbrowser is not JavaScript aware! For
that, you need a real browser. Look at zope.testbrowser.real and Selenium
if you require "real" browser testing.
"""

import unittest
import doctest

from Testing import ZopeTestCase as ztc

from Products.Mlango.tests import base

def test_suite():
    """This sets up a test suite that actually runs the tests in the class
    above
    """
    
    return unittest.TestSuite([

        # Here, we create a test suite passing the name of a file relative 
        # to the package home, the name of the package, and the test base 
        # class to use. Here, the base class is a full PloneTestCase, which
        # means that we get a full Plone site set up.

        # The actual test is in functional.txt

        ztc.ZopeDocFileSuite(
            'tests/functional.txt', package='Products.Mlango',
            test_class=base.MlangoFunctionalTestCase,
            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE | doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS),
            
        # We could add more doctest files here as well, by copying the file
        # block above.

        ])



# -*- coding: utf-8 -*-
#
# File: MlangoDashboard.py
#
# Copyright (c) 2008 by Goldmund-Wyldebeast & Wunderliebe
# Generator: ArchGenXML Version 2.1
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#

__author__ = """Goldmund-Wyldebeast & Wunderliebe <info@gw20e.com>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from zope.interface import implements
import interfaces

from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin

from Products.Mlango.config import *

##code-section module-header #fill in your manual code here
##/code-section module-header

schema = Schema((

    IntegerField(
        name='cols',
        default=4,
        widget=IntegerField._properties['widget'](
            label="Number of columns",
            description="How many columns does the dashboard have?",
            label_msgid="cols_label",
            description_msgid="cols_help",
            i18n_domain="mlango",
        ),
        required=True,
    ),

),
)

##code-section after-local-schema #fill in your manual code here
##/code-section after-local-schema

MlangoDashboard_schema = BaseSchema.copy() + \
    schema.copy()

##code-section after-schema #fill in your manual code here
##/code-section after-schema

class MlangoDashboard(BaseContent, BrowserDefaultMixin):
    """
    """
    security = ClassSecurityInfo()

    implements(interfaces.IMlangoDashboard)

    meta_type = 'MlangoDashboard'
    _at_rename_after_creation = True

    schema = MlangoDashboard_schema

    ##code-section class-header #fill in your manual code here
    aliases = {
        '(Default)'  : 'mlango_dashboard_view', # for now, use the dashboard_view by default
        'view'       : 'mlango_dashboard_view',
        'index.html' : 'mlango_dashboard_view',
        'edit'       : 'atct_edit',
        'properties' : 'base_metadata',
        'sharing'    : 'folder_localrole_form',
        'gethtml'    : '',
        'mkdir'      : '',
        }

    ##/code-section class-header

    # Methods


registerType(MlangoDashboard, PROJECTNAME)
# end of class MlangoDashboard

##code-section module-footer #fill in your manual code here
##/code-section module-footer






# Copyright (C) 2008  Goldmund, Wyldebeast & Wunderliebe
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
from Products.CMFCore.utils import getToolByName


""" Security for viewlets. The viewlet registry returns a list of
    viewlets. A viewlet kan have the dict key 'roles' with a list
    of roles that are required to show the viewlet.

    Use viewletSecurity as a decorator on a function that uses the 
    viewlet registry to get a list of viewlets and returns the same
    list of viewlets. Viewlet security checks if a user has the 
    appropriate role.
"""

def viewletSecurity(func):
    def secureViewletList(*args, **kwargs):
        """..resistance is futile"""
        context =  args[0]
        unsecureList = func(*args, **kwargs)
        secureList = []
        
        member = context.portal_membership.getAuthenticatedMember()
        memberRoles = member.getRoles()

        # add a viewlet if it has no role key
        secureList +=( [v for v in unsecureList if not context.getViewletInfo(v).get('roles') ] )
        # add a viewlet if the role key matches the users group
        for role in memberRoles:
            secureList +=( [v for v in unsecureList\
                 if context.getViewletInfo(v).get('roles')\
                    and role in context.getViewletInfo(v).get('roles')]  )
        return secureList

    return secureViewletList




## Script (Python) "storeUserSettings"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Store user settings
##

# Copyright (C) 2008  Goldmund, Wyldebeast & Wunderliebe
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

tool = context.portal_mlangotool
user_id = context.portal_membership.getAuthenticatedMember().getId()

if not user_id:
  return

data = {}
minmax = {}
dashboard = ""
for p in context.REQUEST.keys():

  if p.find("col_") != -1:
    data[p] = context.REQUEST.get(p)
  if p.find("min") != -1:
    minmax[p] = context.REQUEST.get(p)
  if p.find("dashboard_id") != -1:
    dashboard_id = context.REQUEST.get(p)

print data

tool.updateUserSettings(user_id, dashboard_id, data)
tool.updateUserSettings(user_id, dashboard_id, minmax)

context.REQUEST.RESPONSE.setHeader('Content-type','text/xml')

print '<xml>stored: %s</xml>' % (data)
print '<xml>stored: %s</xml>' % (minmax)

return printed



# -*- coding: utf-8 -*-
#
# File: MlangoViewlet.py
#
# Copyright (c) 2008 by Goldmund-Wyldebeast & Wunderliebe
# Generator: ArchGenXML Version 2.1
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#

__author__ = """Goldmund-Wyldebeast & Wunderliebe <info@gw20e.com>"""
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import *
from zope.interface import implements
import interfaces

from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin

from Products.Mlango.config import *

# additional imports from tagged value 'import'
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

##code-section module-header #fill in your manual code here
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone.memoize.compress import xhtml_compress
##/code-section module-header

schema = Schema((

    ImageField(
        name='viewleticon',
        default="MlangoTool.gif",
        widget=ImageField._properties['widget'](
            label="Viewlet Icon",
            description="Please upload the image that should be used in the toolbar of the viewlet as Icon.",
            label_msgid='Mlango_label_viewleticon',
            description_msgid='Mlango_help_viewleticon',
            i18n_domain='Mlango',
        ),
        storage=AttributeStorage(),
    ),
    TextField(
        name='help',
        widget=TextAreaWidget(
            label="Viewlet Help",
            description="Enter the help for this viewlet here, it will become visible when people press the help button for the viewlet.",
            label_msgid='Mlango_label_help',
            description_msgid='Mlango_help_help',
            i18n_domain='Mlango',
        ),
    ),
    IntegerField(
        name='fixed',
        default=-1,
        widget=IntegerField._properties['widget'](
            description="In which column should this viewlet be fixed? Default is -1 (not fixed). First column is 0, second column 1.",
            label='Fixed',
            label_msgid='Mlango_label_fixed',
            description_msgid='Mlango_help_fixed',
            i18n_domain='Mlango',
        ),
    ),

),
)

##code-section after-local-schema #fill in your manual code here
##/code-section after-local-schema

MlangoViewlet_schema = BaseSchema.copy() + \
    schema.copy()

##code-section after-schema #fill in your manual code here
##/code-section after-schema

class MlangoViewlet(BaseContent, BrowserDefaultMixin):
    """ Super class for all MlangoViewlets, the fields in this class are common for all viewlets.
    Same for the _viewlettemplate, which is shared by all viewlets as well.
    If no _contenttemplate is defined in a viewlet, it uses the "error" template defined here.
    """
    security = ClassSecurityInfo()

    implements(interfaces.IMlangoViewlet)

    meta_type = 'MlangoViewlet'
    _at_rename_after_creation = True

    schema = MlangoViewlet_schema

    ##code-section class-header #fill in your manual code here

    #_viewlettemplate is used by all MlangoViewlets as there frame.
    _viewlettemplate = ViewPageTemplateFile('MlangoViewlet.pt')

    #_contenttemplate is unique for each viewlet contains way to render their content
    _contenttemplate = ViewPageTemplateFile('MlangoViewlet_error.pt')

    ##/code-section class-header

    # Methods

    security.declarePublic('returnTemplate')
    def returnTemplate(self):
        return self._viewlettemplate

    security.declarePublic('returnContentTemplate')
    def returnContentTemplate(self):
        return self._contenttemplate


registerType(MlangoViewlet, PROJECTNAME)
# end of class MlangoViewlet

##code-section module-footer #fill in your manual code here
##/code-section module-footer






"""This is an integration "unit" test. It uses PloneTestCase, but does not
use doctest syntax.

You will find lots of examples of this type of test in CMFPlone/tests, for 
example.
"""

import unittest

from Products.ResourceRegistries.config import CSSTOOLNAME, JSTOOLNAME

from Products.Mlango.tests.base import MlangoTestCase
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import _createObjectByType

from Products.Mlango.config import TOOL_ID
from Products.Mlango.tools.MlangoTool import MlangoTool



class TestMlangoTool(MlangoTestCase):
    """The name of the class should be meaningful. This may be a class that
    tests the installation of a particular product.
    """
    
    def afterSetUp(self):
        self.tool = getattr(self.portal.aq_explicit, TOOL_ID)
        self.viewletId = 'custom-viewlet'
        
        _createObjectByType("MlangoCustomViewlet", self.portal, self.viewletId)
        self.viewlet = getattr(self.portal, self.viewletId)
        self.viewletUID = self.viewlet.UID()
        
        self.viewletData = {'col_0': '%s--' % self.viewletUID, 
                            'col_1': '', 'col_2': '', 'col_3': ''}
                
    def testRegisterViewlet(self):
        """ Register viewlet
        """
        self.tool.registerViewlet(self.viewlet)
 
        self.failUnless(self.viewletUID in self.tool.listViewletIds())

    def testGetViewlet(self):
        """ Get viewlet
        """
        self.failUnless(self.tool.getViewlet(self.viewletUID))
        
    def testUnregisterViewlet(self):
        """ Unregister viewlet
        """
        self.tool.unregisterViewlet(self.viewlet)
        
        self.failIf(self.viewletUID in self.tool.listViewletIds())
                
    def testUserSettings(self):
        """ Store and retrieve user settings
        """
        self.tool.updateUserSettings('dummy-member', 'dashboard' , self.viewletData)

        settingsGlobal = self.tool.getUserSettings('dummy-member', 'dashboard')
        settingsColZero = self.tool.getUserSettings('dummy-member', 'dashboard', 'col_0')
        
        self.assertEqual(settingsGlobal, self.viewletData)
        
        viewletDataCol = self.viewletData.copy()
        del viewletDataCol['col_1']
        del viewletDataCol['col_2']
        del viewletDataCol['col_3']
        self.assertEqual(settingsColZero, viewletDataCol)
        
        self.tool.setUserSettings('dummy-member', 'dashboard' , viewletDataCol)
        settingsGlobalAltered = self.tool.getUserSettings('dummy-member', 'dashboard')
        self.assertEqual(settingsGlobalAltered, viewletDataCol)
        
    def testListUsers(self):
        """ List stored users
        """
        self.tool.updateUserSettings('dummy-member1', 'dashboard' , self.viewletData)
        self.tool.updateUserSettings('dummy-member2', 'dashboard' , self.viewletData)
        
        listUsers = self.tool.listUsers()
        self.assertEqual(len(listUsers), 2)
        self.failUnless('dummy-member1' in listUsers)
        self.failUnless('dummy-member2' in listUsers)        
    
    def testListBoardSettings(self):
        """ List of dashboard(s)
        """
        self.tool.updateUserSettings('dummy-member', 'dashboard' , self.viewletData)
        self.assertEqual(self.tool.listAllBoardsWithSettings(), ['dashboard',])
        
        

    def testFixedViewlet(self):
        """ Get fixed viewlets
        """
        viewletId = 'fixed-viewlet'
        _createObjectByType("MlangoCustomViewlet", self.portal, viewletId)
        viewlet = getattr(self.portal, viewletId)
        viewlet.setFixed(0)
        viewlet.reindexObject()
        self.tool.registerViewlet(viewlet)
        
        self.tool.listViewletIdsByFixedProperty(0)
    
    
def test_suite():
    """This sets up a test suite that actually runs the tests in the class
    above
    """
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestMlangoTool))
    return suite



# -*- coding: utf-8 -*-
#
# File: MlangoTool.py
#
# Copyright (c) 2008 by Goldmund-Wyldebeast & Wunderliebe
# Generator: ArchGenXML Version 2.1
#            http://plone.org/products/archgenxml
#
# GNU General Public License (GPL)
#

__author__ = """Goldmund-Wyldebeast & Wunderliebe <info@gw20e.com>"""
__docformat__ = 'plaintext'

##code-section module-header #fill in your manual code here
import os
from UserSettingsStorage import UserSettingsStorage
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.CMFCore.permissions import *
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.Mlango.config import *
from Globals import PersistentMapping
import interfaces
from Products.CMFCore.utils import getToolByName
##/code-section module-header

from zope.interface import implements
from OFS.SimpleItem import SimpleItem
from Products.CMFCore.utils import UniqueObject
from persistent import Persistent
from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin


class MlangoTool(SimpleItem,UniqueObject,Persistent):
    """
    """

    ##code-section class-header_MlangoTool #fill in your manual code here


    _www = os.path.join(os.path.dirname(__file__), '../www')

    meta_type = 'MlangoTool'
    _at_rename_after_creation = True

    isPrincipiaFolderish = True # Show up in the ZMI
    security = ClassSecurityInfo()

    manage_options = (
                      {'label': 'Info', 'action': 'info'},
                      {'label': 'Viewlets', 'action' : 'manage_viewlets'},
                      {'label': 'Users', 'action' : 'manage_users'},
                      )

    security.declareProtected(ManagePortal, 'manage_viewlets')
    manage_viewlets = PageTemplateFile('manage_viewlets', _www)

    security.declareProtected(ManagePortal, 'manage_users')
    manage_users = PageTemplateFile('manage_users', _www)

    security.declareProtected(View, 'info')
    info = PageTemplateFile('info', _www)

    ##/code-section class-header_MlangoTool


    def __init__(self, id=None):
        self._viewletregistry = {}
        self._usersettingsstorage = UserSettingsStorage()

    def registerViewlet(self, viewlet):
        self._viewletregistry[viewlet.UID()] = {'fixed': viewlet.getFixed()}
        self._changes()

    def unregisterViewlet(self, viewlet):
        if viewlet.UID() in self._viewletregistry.keys():
            del self._viewletregistry[viewlet.UID()]
            self._changes()

    def listViewletIds(self):
        return self._viewletregistry.keys()

    def getViewlet(self, id):
        tool = getToolByName(self, "uid_catalog")
        list = tool(UID=id)
        if len(list) > 0:
            return list[0].getObject()
        else:
            return None

    def listUsers(self):
        return self._usersettingsstorage.keys()

    def getUserSettings(self, user_id, dashboard_id, prefix=None):
        return self._usersettingsstorage.getUserSettings(user_id, dashboard_id, prefix=prefix)

    def setUserSettings(self, user_id, dashboard_id, data):
        return self._usersettingsstorage.setUserSettings(user_id, dashboard_id, data)

    def updateUserSettings(self, user_id, dashboard_id, data):
        return self._usersettingsstorage.updateUserSettings(user_id, dashboard_id, data)

    def listViewletIdsByFixedProperty(self, colnumber):
        """ Find viewlets by a given property """
        return [key for key,value in self._viewletregistry.iteritems() if value['fixed'] == colnumber]

    def listAllBoardsWithSettings(self):
        return self._usersettingsstorage.listAllBoardsWithSettings()

    def listBoardSettings(self, uid):
        return self._usersettingsstorage.listBoardsForUser(uid)


    def at_post_edit_script(self):
        self.unindexObject()


    def _changes(self):
        self._p_changed = 1 # for persistence..


##code-section module-footer #fill in your manual code here

def registerAddedViewlet(viewlet, event):
    """Event handler for edited (and new) viewlets, when a contet type is made,
    this even is called several times. So we wait for it to leave the portal_factory
    before registering, to prevent ghost viewlets to be present.
    """
    if viewlet.absolute_url().find("portal_factory") == -1:
        tool = getToolByName(viewlet, "portal_mlangotool")
        tool.registerViewlet(viewlet)

def unregisterDeletedViewlet(viewlet, event):
    """Event handler for deleting viewlets, unregisters a viewlet.
    """
    tool = getToolByName(viewlet, "portal_mlangotool")
    tool.unregisterViewlet(viewlet)


InitializeClass(MlangoTool)

##/code-section module-footer


